
//==== Includes

//-- sytem

#include <iostream>
using std::cout;
using std::cerr;
using std::endl;

//-- self

#include "Types.h"
#include "str.h"
#include "data.h"
#include "io.h"
#include "ti.h"


//==== Types

//-- structs

struct settings
{
	char* name;
	char* ifpath;
	char* ofpath;

	char* comment;
	bool archived;
	byte version;
	byte type;

	bool reverse;
};


//==== Functions

//-- shortcuts

#define raiseError(msg) cerr << msg << endl
#define raiseSuccess(msg) cout << msg << endl

//-- declarations

int main(int argc, char* argv[]);
bool parseargs(int argc, char* argv[], settings& args);
void freeargs(settings& args);

//-- definitions


int main(int argc, char* argv[])
{
	settings args;
	if ((!parseargs(argc, argv, args)))
	{
		raiseError("Bad parameters.");
		freeargs(args);
		return 1;
	}

	TIFileInfo info;
	{
		if (args.comment != nullptr) lstrset(info.comment, args.comment);
		info.type = args.type;
		if (args.name != nullptr) lstrset(info.name, args.name);
		info.version = args.version;
		info.archived = args.archived;
	}

	data din;
	data dout;

	if (!readfile(args.ifpath, din))
	{
		raiseError("Can't read input file.");
		freeargs(args);
		return 1;
	}

	if (!(args.reverse ? binfrom8x(info, din, dout) : binto8x(info, din, dout)))
	{
		raiseError("Conversion failed.");
		freeargs(args);
		return 1;
	}

	if (!writefile(args.ofpath, dout))
	{
		raiseError("Can't write output file.");
		freeargs(args);
		return 1;
	}

	raiseSuccess("Conversion succeeded.");
	freeargs(args);
	return 0;
}

void freeargs(settings& args)
{
	strdelete(args.name);
	strdelete(args.ifpath);
	strdelete(args.ofpath);
	strdelete(args.comment);
}
bool parseargs(int argc, char* argv[], settings& args)
{
	args.name = nullptr;
	args.ifpath = nullptr;
	args.ofpath = nullptr;
	args.comment = nullptr;
	args.archived = false;
	args.version = 0;
	args.type = tiProtPrgm;
	args.reverse = false;

	const char* argfpath = nullptr;

	for(int i=1; i < argc; ++i){
		const char* arg = argv[i];
		const char* val;

		if (*arg == '-')
		{
			arg++;

			if (++i < argc)
			{
				val = argv[i];
			}
			else
			{
				//parameter value missing
				return false;
			}

			if (strequal(arg, "if"))
			{
				args.ifpath = strcopy(val);
			}
			else if (strequal(arg, "of"))
			{
				args.ofpath = strcopy(val);
			}
			else if (strequal(arg, "mem"))
			{
				//ram|arc
				if (strequal(val, "ram"))
					args.archived = false;
				else if (strequal(val, "arc"))
					args.archived = true;
				else
					return false;
			}
			else if (strequal(arg, "type"))
			{
				//(prot)prgm|appv
				if (strequal(val, "prgm"))
					args.type = tiPrgm;
				else if(strequal(val, "protprgm"))
					args.type = tiProtPrgm;
				else if (strequal(val, "appv"))
					args.type = tiAppv;
				else
					return false;
			}
			else if (strequal(arg, "version"))
			{
				//version
				args.version = byte(str_to_int(val));
			}
			else if (strequal(arg, "comment"))
			{
				//comment
				args.comment = strcopy(val);
			}
			else if (strequal(arg, "mode"))
			{
				//bin(from|to)8x
				if (strequal(val, "from"))
					args.reverse = true;
				else if (strequal(val, "to"))
					args.reverse = false;
				else
					return false;
			}
			else
			{
				//unknown parameter
				return false;
			}
		}
		else
		{
			if (args.name == nullptr)
			{
				args.name = strcopy(arg);
			}
			else if (argfpath == nullptr)
			{
				argfpath = arg;
			}
			else
			{
				//too many args
				return false;
			}
		}
	}

	const char *ext;
	switch (args.type)
	{
		case tiPrgm:
		case tiProtPrgm:
			ext = ".8xp"; break;
		case tiAppv:
			ext = ".8xv"; break;
		default:
			ext = ".8x_"; break;
	}

	bool argsname_empty = (args.name == nullptr);
	bool argfpath_empty = (argfpath == nullptr);
	bool argspath_need = false;

	char* pathname = strnew(argfpath_empty ? "" : argfpath, argsname_empty ? "" : args.name);
	if (args.ifpath == nullptr)
	{
		argspath_need = true;
		args.ifpath = strnew(pathname, args.reverse ? ext : ".bin");
	}
	if (args.ofpath == nullptr)
	{
		argspath_need = true;
		args.ofpath = strnew(pathname, args.reverse ? ".bin" : ext);
	}
	strdelete(pathname);

	return !(((argspath_need && argfpath_empty) || (!args.reverse)) && argsname_empty);
}
