#ifndef DATA_H
#define DATA_H

//==== Includes

//-- sytem

//-- self

#include "Types.h"


//==== Class

class data
{
public:
	data(size_t capacity = 65536);
	~data();

	size_t pos() const;
	size_t size() const;
	size_t capacity() const;

	size_t pos(size_t pos);

	explicit operator bool() const;

	data& operator << (const data& put);
	const data& operator >> (data& get) const;
	data& operator >> (data& get);

	data& operator << (byte put);
	data& operator >> (byte& get);

	data& operator << (uint put);
	data& operator >> (uint& get);

	template<size_t L> data& operator << (char (&text)[L]);
	template<size_t L> data& operator >> (char (&text)[L]);

	data& strput(const char* text, size_t len);
	data& strget(char* text, size_t len);

private:
	byte* const bytes;
	size_t pos_;
	size_t const capacity_;
	size_t size_;
public:
	bool fail;
	bool error;
};


inline data::data(size_t capacity)
: fail(false), error(false), pos_(0), size_(0), capacity_(capacity), bytes(new byte[capacity])
{
}
inline data::~data()
{
	delete[] bytes;
}

inline size_t data::pos() const
{
	return pos_;
}
inline size_t data::size() const
{
	return size_;
}
inline size_t data::capacity() const
{
	return capacity_;
}

inline size_t data::pos(size_t pos)
{
	size_t old = pos_;
	pos_ = pos;
	return old;
}

inline data::operator bool() const
{
	return !(fail || error);
}

inline data& data::operator << (const data& put)
{
	if (this != &put)
	{
		for (size_t pos = 0; pos < put.size_; pos++)
		{
			*this << put.bytes[pos];
		}
	}
	else
	{
		error = true;
	}
	return *this;
}
inline const data& data::operator >> (data& get) const
{
	get << *this;
	return *this;
}
inline data& data::operator >> (data& get)
{
	get << *this;
	return *this;
}

inline data& data::operator << (byte put)
{
	if (size_ < capacity_)
	{
		bytes[size_++] = put;
	}
	else
	{
		error = true;
	}
	return *this;
}
inline data& data::operator >> (byte& get)
{
	if (pos_ < size_)
	{
		get = bytes[pos_++];
	}
	else
	{
		fail = true;
	}
	return *this;
}

inline data& data::operator << (uint put)
{
	/*
	byte lsb = byte(put % 256);
	byte msb = byte((put / 256) % 256);
	if (put > 65535) error = true;
	return (*this << lsb << msb);
	*/

	if ((size_+1 < capacity_) && (put < 65536))
	{
		bytes[size_++] = byte(put % 256);
		bytes[size_++] = byte((put / 256) % 256);
	}
	else
	{
		error = true;
	}
	return *this;
}
inline data& data::operator >> (uint& get)
{
	/*
	byte lsb; byte msb;
	*this >> lsb >> msb;
	get = msb * 256 + lsb;
	return *this;
	*/

	if (pos_+1 < size_)
	{
		get = bytes[pos_] + (bytes[pos_+1] * 256);
		pos_ += 2;
	}
	else
	{
		fail = true;
	}
	return *this;
}

template<size_t L>
inline data& data::operator << (char (&text)[L])
{
	return strput(text, L);
}

template<size_t L>
inline data& data::operator >> (char (&text)[L])
{
	return strget(text, L);
}

inline data& data::strput(const char* text, size_t len)
{
	while (len --> 0)
	{
		if (size_ >= capacity_)
		{
			error = true;
			break;
		}

		char c = *text++;
		byte b = byte_to_char(c);
		bytes[size_++] = b;
	}

	return *this;
}
inline data& data::strget(char* text, size_t len)
{
	while (len --> 0)
	{
		if (pos_ >= size_)
		{
			fail = true;
			break;
		}

		byte b = bytes[pos_++];
		char c = byte_to_char(b);
		*text++ = c;
	}

	return *this;
}

#endif /* DATA_H */