
//==== Includes

//-- sytem

//-- self

#include "Types.h"
#include "str.h"
#include "data.h"
#include "ti.h"


//==== Functions

uint checksum(data& data, size_t beg, size_t end)
{
	byte value;
	uint sum = 0;
	size_t old = data.pos(beg);
	while (data.pos() != end)
	{
		data >> value;
		sum += value;
	}
	data.pos(old);
	return sum % 65536;
}

bool binto8x(TIFileInfo& info, data& in, data& out)
{
	//HEADER

	//signature
	out.strput("**TI83F*", 8);
	out << byte(26); out << byte(10); out << byte(0);

	//comment
	lstrpad(info.comment, ' ');
	out << info.comment;

	//length of file data section
	out << uint(in.size() + 19);

	size_t checksumStart = out.size();


	//DATA SECTION

	//eleven or thirteen
	out << uint(info.entry); //11 or 13

	//length of variable data
	out << uint(in.size() + 2);

	//variable type
	out << byte(info.type);

	//variable name
	lstrpad(info.name, '\0');
	lstrreplace(info.name, '_', byte_to_char(91));
	out << info.name;

	//version
	out << info.version;

	//flag
	out << byte(info.archived ? 128 : 0);

	//length of variable data
	out << uint(in.size() + 2);

	//variable data
	out << uint(in.size());
	out << in;


	// FOOTER

	size_t checksumEnd = out.size();

	//data section checksum
	out << checksum(out, checksumStart, checksumEnd);


	return bool(out);
}

bool binfrom8x(TIFileInfo& info, data& in, data& out)
{
	char tempstr8[8];
	uint tempuint;
	byte tempbyte;

	uint size;


	//HEADER

	//signature
	in >> tempstr8; if (!lstrequal(tempstr8, "**TI83F*")) return false;
	in >> tempbyte; if (tempbyte != 26) return false;
	in >> tempbyte; if (tempbyte != 10) return false;
	in >> tempbyte; if (tempbyte != 0) return false;

	//comment
	in.strget(info.comment, 42);

	//length of file data section
	in >> tempuint; size = tempuint - 19;

	size_t checksumStart = in.pos();


	//DATA SECTION

	//eleven or thirteen
	in >> tempuint; if ((tempuint != 11) && (tempuint != 13)) return false;
	info.entry = byte(tempuint);

	//length of variable data
	in >> tempuint; if (tempuint != size + 2) return false;

	//variable type
	in >> tempbyte;
	info.type = tempbyte;

	//variable name
	in >> info.name; lstrreplace(info.name, byte_to_char(91), '_');

	//version
	in >> info.version;

	//flag
	in >> tempbyte;
	info.archived = ((tempbyte & 128) != 0);

	//length of variable data
	in >> tempuint; if (tempuint != size + 2) return false;

	//variable data
	in >> tempuint; if (tempuint != size) return false;
	for (uint i = 0; i < size; i++)
	{
		in >> tempbyte;
		out << tempbyte;
	}


	// FOOTER

	size_t checksumEnd = in.pos();
	uint sum = checksum(in, checksumStart, checksumEnd);

	//data section checksum
	in >> tempuint; if (tempuint != sum) return false;

	return bool(in);
}
